@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2008, 2010, 2012, 2013 Richard Dymond (rjdymond@gmail.com)
+# Copyright 2008, 2010, 2012-2014 Richard Dymond (rjdymond@gmail.com)
 #
 # This file is part of Pyskool.
 #
@@ -50,6 +50,8 @@ QUIT = 'QUIT'
 SCALE_UP = 'SCALE_UP'
 #: Menu operation: Decrease scale.
 SCALE_DOWN = 'SCALE_DOWN'
+#: Menu operation: Toggle fullscreen mode.
+TOGGLE_FULLSCREEN = 'TOGGLE_FULLSCREEN'
 #: Menu operation: Update the screen.
 UPDATE = 'UPDATE'
 
@@ -79,11 +81,16 @@ class Game:
             sys.stdout.write("WARNING: pygame.mixer failed to initialise; there will be no sound\n")
 
         parser = IniParser(ini_file)
-        menu_name = 'Main'
-        self.menu_config = parser.get_config('Menu %s' % menu_name)
-        self.menu_items = parser.parse_section('MenuItems %s' % menu_name)
         self._create_key_bindings(parser.parse_section('Keys'))
 
+        self.menus = {}
+        self.menu_config = {}
+        for menu_name in ('Main', 'Quit'):
+            self.menu_config[menu_name] = (
+                parser.get_config('Menu {0}'.format(menu_name)),
+                parser.parse_section('MenuItems {0}'.format(menu_name))
+            )
+
         self.images_dir = images_dir
         self.sounds_dir = sounds_dir
         self.scale = scale
@@ -91,20 +98,24 @@ class Game:
         self.screenshot = 0
         self.version = version
         self.screen = None
-        self.menu_up = False
+        self.menu = None
+
+        builder = skoolbuilder.SkoolBuilder(ini_dir)
+        config = builder.get_config('[A-Za-z]+Config')
+        self.cheat = cheat or config.get('Cheat', 0)
+        self.quick_start = quick_start or config.get('QuickStart', 0)
+        self.ring_bell = not self.quick_start
+        self.confirm_close = config.get('ConfirmClose', 0)
+        self.confirm_quit = config.get('ConfirmQuit', 1)
 
         if sav_file:
             if os.path.isfile(sav_file):
                 self.quick_start = True
                 self.ring_bell = True
-                self.cheat = cheat
                 self._load(sav_file)
                 return
             debug.log('Unable to restore from %s: file not found' % sav_file)
 
-        builder = skoolbuilder.SkoolBuilder(ini_dir)
-        config = builder.get_config('[A-Za-z]+Config')
-
         gallery = Gallery(images_dir, config, scale, builder.get_config(skoolbuilder.IMAGES))
         title_prefix = 'Pyskool %s: ' % version
         self.screen = Screen(config, gallery, title_prefix)
@@ -117,19 +128,19 @@ class Game:
         self.screen.initialise_column(self.skool.get_width(), self.cast.eric.x)
         self.screen.setup()
         self.skool.update_score_box()
+        self._build_menus()
 
-        self.cheat = cheat or config.get('Cheat', 0)
-        self.quick_start = quick_start or config.get('QuickStart', 0)
-        self.ring_bell = not self.quick_start
+    def _build_menus(self):
+        """Build the menus."""
+        for menu_name, (menu_config, menu_items) in self.menu_config.items():
+            self.menus[menu_name] = Menu(self.screen, menu_config, menu_items)
 
-        self._build_menu()
-
-    def _build_menu(self):
-        """Build the menu."""
-        self.menu = Menu(self.screen, self.menu_config, self.menu_items)
+    def _build_menu_images(self):
+        for menu in self.menus.values():
+            menu.build_images(self.screen)
 
     def _handle_menu(self):
-        """Handle keypresses while the menu is displayed."""
+        """Handle keypresses while a menu is displayed."""
         draw = False
         refresh = False
         status = ''
@@ -137,7 +148,8 @@ class Game:
         if operation == UPDATE:
             draw = True
         elif operation == RESUME:
-            self.menu_up = False
+            self.menu = None
+            self.skool.draw()
         elif operation == SAVE:
             sav_file = self._save_game()
             status = 'Saved %s' % sav_file
@@ -152,14 +164,16 @@ class Game:
             self.skool.draw(False)
             refresh = True
             status = 'Scale set to %i' % self.screen.scale
-            self.menu.build_images(self.screen)
+            self._build_menu_images()
         elif operation == SCALE_DOWN:
             if self.screen.scale_down():
                 self.skool.restore()
                 self.skool.draw(False)
                 refresh = True
                 status = 'Scale set to %i' % self.screen.scale
-                self.menu.build_images(self.screen)
+                self._build_menu_images()
+        elif operation == TOGGLE_FULLSCREEN:
+            pygame.display.toggle_fullscreen()
         elif operation == QUIT:
             return True
         if draw or status:
@@ -230,7 +244,7 @@ class Game:
         self.skool.beeper.restore(self.sounds_dir)
         self.skool.gallery.restore(self.images_dir)
         self.skool.restore()
-        self._build_menu()
+        self._build_menus()
 
         debug.log('Skool loaded from %s in %0.2fs' % (os.path.abspath(fname), time.time() - start))
 
@@ -386,13 +400,8 @@ class Game:
         * update the screen
         * scroll the screen if necessary
 
-        :return: `True` if the quit key was pressed, 'Quit' was selected in the
-                 menu, or the window close button was clicked; `False`
-                 otherwise.
+        :return: `True` if the game is quitting, `False` otherwise.
         """
-        if self.keyboard.got_quit():
-            return True
-
         if self.keyboard.was_pressed(keys.FULL_SCREEN):
             pygame.display.toggle_fullscreen()
             return False
@@ -400,11 +409,24 @@ class Game:
         if self.keyboard.was_pressed(keys.SCREENSHOT):
             self._take_screenshot()
 
-        if self.menu_up:
+        if self.menu:
             return self._handle_menu()
 
-        if self.keyboard.was_pressed(keys.QUIT, force_check=True):
-            return True
+        show_quit_menu = False
+        if self.keyboard.got_quit():
+            if not self.confirm_close:
+                return True
+            show_quit_menu = True
+        elif self.keyboard.was_pressed(keys.QUIT, force_check=True):
+            if not self.confirm_quit:
+                return True
+            show_quit_menu = True
+        if show_quit_menu:
+            self.menu = self.menus['Quit']
+            self.menu.reset()
+            self.screen.draw_menu(self.menu)
+            self.beeper.pause()
+            return False
 
         self.paused ^= self.keyboard.was_pressed(keys.PAUSE, force_check=True)
         if self.paused:
@@ -447,9 +469,9 @@ class Game:
         self.skool.scroll(self.scroll, self.clock)
 
         if self.keyboard.was_pressed(keys.MENU):
+            self.menu = self.menus['Main']
             self.menu.reset()
             self.screen.draw_menu(self.menu)
-            self.menu_up = True
         elif self.keyboard.was_pressed(keys.SAVE):
             self._save_game()
         elif self.keyboard.was_pressed(keys.LOAD):
@@ -470,6 +492,7 @@ class Menu:
         self.ink = config.get('Ink', (255, 255, 255))
         self.paper = config.get('Paper', (255, 0, 0))
         self.highlight = config.get('Highlight', (200, 0, 0))
+        self.status_bar = config.get('StatusBar', 1)
         self.status_paper = config.get('StatusPaper', (100, 100, 100))
         self.title_paper = config.get('TitlePaper', (100, 100, 100))
         self.width = config.get('Width', 0.9)
